@@ -11,7 +11,7 @@ func (store *EtcdStore) KvPut(ctx context.Context, key []byte, value []byte) (er
 	_, err = store.client.Put(ctx, store.etcdKeyPrefix+string(key), string(value))
 
 	if err != nil {
-		return fmt.Errorf("kv put: %v", err)
+		return fmt.Errorf("kv put: %w", err)
 	}
 
 	return nil
@@ -22,7 +22,7 @@ func (store *EtcdStore) KvGet(ctx context.Context, key []byte) (value []byte, er
 	resp, err := store.client.Get(ctx, store.etcdKeyPrefix+string(key))
 
 	if err != nil {
-		return nil, fmt.Errorf("kv get: %v", err)
+		return nil, fmt.Errorf("kv get: %w", err)
 	}
 
 	if len(resp.Kvs) == 0 {
@@ -37,7 +37,7 @@ func (store *EtcdStore) KvDelete(ctx context.Context, key []byte) (err error) {
 	_, err = store.client.Delete(ctx, store.etcdKeyPrefix+string(key))
 
 	if err != nil {
-		return fmt.Errorf("kv delete: %v", err)
+		return fmt.Errorf("kv delete: %w", err)
 	}
 
 	return nil
