@@ -13,7 +13,7 @@ func (store *LevelDB3Store) KvPut(ctx context.Context, key []byte, value []byte)
 	err = store.dbs[DEFAULT].Put(key, value, nil)
 
 	if err != nil {
-		return fmt.Errorf("kv put: %v", err)
+		return fmt.Errorf("kv put: %w", err)
 	}
 
 	return nil
@@ -28,7 +28,7 @@ func (store *LevelDB3Store) KvGet(ctx context.Context, key []byte) (value []byte
 	}
 
 	if err != nil {
-		return nil, fmt.Errorf("kv get: %v", err)
+		return nil, fmt.Errorf("kv get: %w", err)
 	}
 
 	return
@@ -39,7 +39,7 @@ func (store *LevelDB3Store) KvDelete(ctx context.Context, key []byte) (err error
 	err = store.dbs[DEFAULT].Delete(key, nil)
 
 	if err != nil {
-		return fmt.Errorf("kv delete: %v", err)
+		return fmt.Errorf("kv delete: %w", err)
 	}
 
 	return nil
