@@ -15,7 +15,7 @@ func (store *RocksDBStore) KvPut(ctx context.Context, key []byte, value []byte)
 	err = store.db.Put(store.wo, key, value)
 
 	if err != nil {
-		return fmt.Errorf("kv put: %v", err)
+		return fmt.Errorf("kv put: %w", err)
 	}
 
 	return nil
@@ -30,7 +30,7 @@ func (store *RocksDBStore) KvGet(ctx context.Context, key []byte) (value []byte,
 	}
 
 	if err != nil {
-		return nil, fmt.Errorf("kv get: %v", err)
+		return nil, fmt.Errorf("kv get: %w", err)
 	}
 
 	return
@@ -41,7 +41,7 @@ func (store *RocksDBStore) KvDelete(ctx context.Context, key []byte) (err error)
 	err = store.db.Delete(store.wo, key)
 
 	if err != nil {
-		return fmt.Errorf("kv delete: %v", err)
+		return fmt.Errorf("kv delete: %w", err)
 	}
 
 	return nil
