@@ -91,11 +91,12 @@ func NewStore(grpcDialOption grpc.DialOption, ip string, port int, grpcPort int,
 		s.Locations = append(s.Locations, location)
 		stats.VolumeServerMaxVolumeCounter.Add(float64(maxVolumeCounts[i]))
 
+		diskId := uint32(i) // Track disk ID
 		wg.Add(1)
-		go func() {
+		go func(id uint32, diskLoc *DiskLocation) {
 			defer wg.Done()
-			location.loadExistingVolumes(needleMapKind, ldbTimeout)
-		}()
+			diskLoc.loadExistingVolumesWithId(needleMapKind, ldbTimeout, id)
+		}(diskId, location)
 	}
 	wg.Wait()
 
@@ -107,7 +108,7 @@ func NewStore(grpcDialOption grpc.DialOption, ip string, port int, grpcPort int,
 
 	return
 }
-func (s *Store) AddVolume(volumeId needle.VolumeId, collection string, needleMapKind NeedleMapKind, replicaPlacement string, ttlString string, preallocate int64, MemoryMapMaxSizeMb uint32, diskType DiskType, ldbTimeout int64) error {
+func (s *Store) AddVolume(volumeId needle.VolumeId, collection string, needleMapKind NeedleMapKind, replicaPlacement string, ttlString string, preallocate int64, ver needle.Version, MemoryMapMaxSizeMb uint32, diskType DiskType, ldbTimeout int64) error {
 	rt, e := super_block.NewReplicaPlacementFromString(replicaPlacement)
 	if e != nil {
 		return e
@@ -116,7 +117,7 @@ func (s *Store) AddVolume(volumeId needle.VolumeId, collection string, needleMap
 	if e != nil {
 		return e
 	}
-	e = s.addVolume(volumeId, collection, needleMapKind, rt, ttl, preallocate, MemoryMapMaxSizeMb, diskType, ldbTimeout)
+	e = s.addVolume(volumeId, collection, needleMapKind, rt, ttl, preallocate, ver, MemoryMapMaxSizeMb, diskType, ldbTimeout)
 	return e
 }
 func (s *Store) DeleteCollection(collection string) (e error) {
@@ -159,18 +160,33 @@ func (s *Store) FindFreeLocation(filterFn func(location *DiskLocation) bool) (re
 	}
 	return ret
 }
-func (s *Store) addVolume(vid needle.VolumeId, collection string, needleMapKind NeedleMapKind, replicaPlacement *super_block.ReplicaPlacement, ttl *needle.TTL, preallocate int64, memoryMapMaxSizeMb uint32, diskType DiskType, ldbTimeout int64) error {
+func (s *Store) addVolume(vid needle.VolumeId, collection string, needleMapKind NeedleMapKind, replicaPlacement *super_block.ReplicaPlacement, ttl *needle.TTL, preallocate int64, ver needle.Version, memoryMapMaxSizeMb uint32, diskType DiskType, ldbTimeout int64) error {
 	if s.findVolume(vid) != nil {
 		return fmt.Errorf("Volume Id %d already exists!", vid)
 	}
-	if location := s.FindFreeLocation(func(location *DiskLocation) bool {
-		return location.DiskType == diskType
-	}); location != nil {
-		glog.V(0).Infof("In dir %s adds volume:%v collection:%s replicaPlacement:%v ttl:%v",
-			location.Directory, vid, collection, replicaPlacement, ttl)
-		if volume, err := NewVolume(location.Directory, location.IdxDirectory, collection, vid, needleMapKind, replicaPlacement, ttl, preallocate, memoryMapMaxSizeMb, ldbTimeout); err == nil {
+
+	// Find location with lowest local volume count (load balancing)
+	var location *DiskLocation
+	var diskId uint32
+	var minVolCount int
+	for i, loc := range s.Locations {
+		if loc.DiskType == diskType && s.hasFreeDiskLocation(loc) {
+			volCount := loc.LocalVolumesLen()
+			if location == nil || volCount < minVolCount {
+				location = loc
+				diskId = uint32(i)
+				minVolCount = volCount
+			}
+		}
+	}
+
+	if location != nil {
+		glog.V(0).Infof("In dir %s (disk ID %d) adds volume:%v collection:%s replicaPlacement:%v ttl:%v",
+			location.Directory, diskId, vid, collection, replicaPlacement, ttl)
+		if volume, err := NewVolume(location.Directory, location.IdxDirectory, collection, vid, needleMapKind, replicaPlacement, ttl, preallocate, ver, memoryMapMaxSizeMb, ldbTimeout); err == nil {
+			volume.diskId = diskId // Set the disk ID
 			location.SetVolume(vid, volume)
-			glog.V(0).Infof("add volume %d", vid)
+			glog.V(0).Infof("add volume %d on disk ID %d", vid, diskId)
 			s.NewVolumesChan <- master_pb.VolumeShortInformationMessage{
 				Id:               uint32(vid),
 				Collection:       collection,
@@ -178,6 +194,7 @@ func (s *Store) addVolume(vid needle.VolumeId, collection string, needleMapKind
 				Version:          uint32(volume.Version()),
 				Ttl:              ttl.ToUint32(),
 				DiskType:         string(diskType),
+				DiskId:           diskId,
 			}
 			return nil
 		} else {
@@ -187,6 +204,22 @@ func (s *Store) addVolume(vid needle.VolumeId, collection string, needleMapKind
 	return fmt.Errorf("No more free space left")
 }
 
+// hasFreeDiskLocation checks if a disk location has free space
+func (s *Store) hasFreeDiskLocation(location *DiskLocation) bool {
+	// Check if disk space is low first
+	if location.isDiskSpaceLow {
+		return false
+	}
+
+	// If MaxVolumeCount is 0, it means unlimited volumes are allowed
+	if location.MaxVolumeCount == 0 {
+		return true
+	}
+
+	// Check if current volume count is below the maximum
+	return int64(location.VolumesLen()) < int64(location.MaxVolumeCount)
+}
+
 func (s *Store) VolumeInfos() (allStats []*VolumeInfo) {
 	for _, location := range s.Locations {
 		stats := collectStatsForOneLocation(location)
@@ -218,6 +251,7 @@ func collectStatForOneVolume(vid needle.VolumeId, v *Volume) (s *VolumeInfo) {
 		Ttl:              v.Ttl,
 		CompactRevision:  uint32(v.CompactionRevision),
 		DiskType:         v.DiskType().String(),
+		DiskId:           v.diskId,
 	}
 	s.RemoteStorageName, s.RemoteStorageKey = v.RemoteStorageNameKey()
 
@@ -384,7 +418,7 @@ func (s *Store) CollectHeartbeat() *master_pb.Heartbeat {
 }
 
 func (s *Store) deleteExpiredEcVolumes() (ecShards, deleted []*master_pb.VolumeEcShardInformationMessage) {
-	for _, location := range s.Locations {
+	for diskId, location := range s.Locations {
 		// Collect ecVolume to be deleted
 		var toDeleteEvs []*erasure_coding.EcVolume
 		location.ecVolumesLock.RLock()
@@ -392,7 +426,7 @@ func (s *Store) deleteExpiredEcVolumes() (ecShards, deleted []*master_pb.VolumeE
 			if ev.IsTimeToDestroy() {
 				toDeleteEvs = append(toDeleteEvs, ev)
 			} else {
-				messages := ev.ToVolumeEcShardInformationMessage()
+				messages := ev.ToVolumeEcShardInformationMessage(uint32(diskId))
 				ecShards = append(ecShards, messages...)
 			}
 		}
@@ -400,7 +434,7 @@ func (s *Store) deleteExpiredEcVolumes() (ecShards, deleted []*master_pb.VolumeE
 
 		// Delete expired volumes
 		for _, ev := range toDeleteEvs {
-			messages := ev.ToVolumeEcShardInformationMessage()
+			messages := ev.ToVolumeEcShardInformationMessage(uint32(diskId))
 			// deleteEcVolumeById has its own lock
 			err := location.deleteEcVolumeById(ev.VolumeId)
 			if err != nil {
@@ -441,7 +475,7 @@ func (s *Store) WriteVolumeNeedle(i needle.VolumeId, n *needle.Needle, checkCook
 			err = fmt.Errorf("volume %d is read only", i)
 			return
 		}
-		_, _, isUnchanged, err = v.writeNeedle2(n, checkCookie, fsync || s.isStopping)
+		_, _, isUnchanged, err = v.writeNeedle2(n, checkCookie, fsync && !s.isStopping)
 		return
 	}
 	glog.V(0).Infoln("volume", i, "not found!")
@@ -515,10 +549,11 @@ func (s *Store) MarkVolumeWritable(i needle.VolumeId) error {
 }
 
 func (s *Store) MountVolume(i needle.VolumeId) error {
-	for _, location := range s.Locations {
-		if found := location.LoadVolume(i, s.NeedleMapKind); found == true {
+	for diskId, location := range s.Locations {
+		if found := location.LoadVolume(uint32(diskId), i, s.NeedleMapKind); found == true {
 			glog.V(0).Infof("mount volume %d", i)
 			v := s.findVolume(i)
+			v.diskId = uint32(diskId) // Set disk ID when mounting
 			s.NewVolumesChan <- master_pb.VolumeShortInformationMessage{
 				Id:               uint32(v.Id),
 				Collection:       v.Collection,
@@ -526,6 +561,7 @@ func (s *Store) MountVolume(i needle.VolumeId) error {
 				Version:          uint32(v.Version()),
 				Ttl:              v.Ttl.ToUint32(),
 				DiskType:         string(v.location.DiskType),
+				DiskId:           uint32(diskId),
 			}
 			return nil
 		}
@@ -546,6 +582,7 @@ func (s *Store) UnmountVolume(i needle.VolumeId) error {
 		Version:          uint32(v.Version()),
 		Ttl:              v.Ttl.ToUint32(),
 		DiskType:         string(v.location.DiskType),
+		DiskId:           v.diskId,
 	}
 
 	for _, location := range s.Locations {
@@ -574,6 +611,7 @@ func (s *Store) DeleteVolume(i needle.VolumeId, onlyEmpty bool) error {
 		Version:          uint32(v.Version()),
 		Ttl:              v.Ttl.ToUint32(),
 		DiskType:         string(v.location.DiskType),
+		DiskId:           v.diskId,
 	}
 	for _, location := range s.Locations {
 		err := location.DeleteVolume(i, onlyEmpty)
