@@ -4,6 +4,6 @@
 package util
 
 const (
-	sizeLimit         = "8000GB"
+	SizeLimit         = "8000GB"
 	VolumeSizeLimitGB = 8000
 )
